@@ -1,7 +1,7 @@
 # John Maloney, October 2022
 # Revised by Wenjie Wu, October 2022
 
-__version__ = "0.7.1"
+__version__ = "0.7.2"
 
 import uuid
 import threading
@@ -159,7 +159,7 @@ class MicroblocksBLEMessage(MicroBlocksBase):
     todo: supports connecting multiple devices
     """
 
-    def __init__(self, device_name=None, verbose=False):
+    def __init__(self, device_name_or_advertisement=None, verbose=False):
         self._ble = BLERadio()
         self.found_devices = {}
         self.connection = None
@@ -167,25 +167,28 @@ class MicroblocksBLEMessage(MicroBlocksBase):
         self._verbose = verbose  # verbose: Print various debugging information
         self.on_message = None  # paho style:  https://www.eclipse.org/paho/index.php?page=clients/python/docs/index.php
 
-        if device_name:
-            self.connect(device_name)
+        if device_name_or_advertisement:
+            self.connect(device_name_or_advertisement)
 
-    def connect(self, device_name, timeout=3):
+    def connect(self, device_name_or_advertisement, timeout=3):
         # MicroBlocks KCY
         # assert len(self._ble.connections) == 0
-        if device_name not in self.found_devices:
-            self.discover(timeout)
-
-        if device_name in self.found_devices:
-            self.connection = self._ble.connect(self.found_devices[device_name])
-            print(f"{device_name} is connected")
+        if type(device_name_or_advertisement) == ProvideServicesAdvertisement:
+            self.connection = self._ble.connect(device_name_or_advertisement)
         else:
-            raise Exception("Device not found")
+            if device_name_or_advertisement not in self.found_devices:
+                self.discover(timeout)
+
+            if device_name_or_advertisement in self.found_devices:
+                self.connection = self._ble.connect(self.found_devices[device_name_or_advertisement])
+                print(f"{device_name_or_advertisement} is connected")
+            else:
+                raise Exception("Device not found")
 
     def discover(self, timeout=3):
         for advertisement in self._ble.start_scan(
             ProvideServicesAdvertisement, timeout=timeout
-        ):
+        ):            
             if MicroBlocksIDEService in advertisement.services:
                 self.found_devices[advertisement.complete_name] = advertisement
         return list(self.found_devices.keys())
@@ -270,14 +273,14 @@ Agent.send = send
 
 
 class MicroblocksClient(MicroblocksBLEMessage):
-    def __init__(self, device_name=None, verbose=False):
-        super().__init__(device_name, verbose)
+    def __init__(self, device_name_or_advertisement=None, verbose=False):
+        super().__init__(device_name_or_advertisement, verbose)
         self.agent = Agent("agent")
         self.agent.microblocks_client = self
         # print("connected:", self.connection.connected)
 
-    def connect(self, device_name, timeout=3):
-        super().connect(device_name, timeout)
+    def connect(self, device_name_or_advertisement, timeout=3):
+        super().connect(device_name_or_advertisement, timeout)
         if self.connection and self.connection.connected:
             self.send("_start BLE loop")  # ??
             time.sleep(0.1)
